--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG and Lily.
-- by rabbit.magtheridon@gmail.com
--
--]]

local statusbarTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\armory"
local borderTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\border"
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
local power = oUF.colors.power

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local grey = {0.5, 0.5, 0.5}
local white = {1, 1, 1}
local name = "%s %s"
local classifications = {
	worldboss = "??",
	rareelite = "%s*+",
	rare = "%s*",
	elite = "%s+",
}
local function updateName(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local u = unit or self.unit
		
		if unit == "target" then
			local cl = UnitClassification(u)
			local level = classifications[cl] and classifications[cl]:format(UnitLevel(u)) or UnitLevel(u)
			local n = UnitName(u)
			self.Name:SetText(name:format(level, n))
		else
			local n = UnitName(u)
			self.Name:SetText(n)
		end

		local c = (not UnitIsConnected(u) or UnitIsGhost(u) or UnitIsDead(u)) and grey or white
		self.Name:SetTextColor(unpack(c))
	end
end

local function updateHealth(self, event, bar, unit, min, max)
	if UnitIsDead(unit) then
		bar.value:SetText("Dead")
		bar.value:SetTextColor(.5, .5, .5)
	elseif UnitIsGhost(unit) then
		bar.value:SetText("Ghost")
		bar.value:SetTextColor(.5, .5, .5)
	elseif not UnitIsConnected(unit) then
		bar.value:SetText("Off")
		bar.value:SetTextColor(.5, .5, .5)
	else
		local value = nil
		if max == 100 then -- Percentage value
			value = min .. "%"
		elseif min ~= max then
			value = "-" .. (max - min)
		elseif unit == "target" then
			value = max
		end
		if value then
			bar.value:SetText(value)
			if max == min then
				bar.value:SetTextColor(1, 1, 1)
			else
				bar.value:SetTextColor(1, 0.5, 0.5)
			end
		else
			bar.value:SetText("")
		end
		if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
			bar:SetStatusBarColor(.5, .5, .3)
		else
			local _, c = UnitClass(unit)
			local color = UnitIsFriend(unit, "player") and UnitIsPlayer(unit) and RAID_CLASS_COLORS[c] or UnitReactionColor[UnitReaction(unit, "player")]
			if color then
				bar:SetStatusBarColor(color.r, color.g, color.b)
			end
			self:UNIT_NAME_UPDATE(event, unit)
		end
	end
end

local function updatePower(self, event, bar, unit, min, max)
	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit) then
		bar:SetStatusBarColor(.6, .6, .6)
	else
		local color = power[UnitPowerType(unit)]
		if color then
			bar:SetStatusBarColor(color.r, color.g, color.b)
		end
	end
end

local function auraIcon(self, button)
	local t = button:CreateTexture(nil, "OVERLAY")
	t:SetTexture(borderTexture)
	t:SetAllPoints(button)
	t:SetVertexColor(0.25, 0.25, 0.35)
end

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

local function func(settings, self, unit)
	self.menu = menu
	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	if not unit then -- Party
		self:SetWidth(168) 
		self:SetHeight(36)
	elseif unit == "targettarget" or unit == "focus" then
		self:SetWidth(92)
		self:SetHeight(20)
	elseif unit == "player" or unit == "target" then
		self:SetWidth(192)
		self:SetHeight(36)
	elseif unit == "pet" then
		self:SetWidth(192)
		self:SetHeight(20)
	end

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, .9)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(20)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetStatusBarColor(0, 1, 0)
	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")

	local hpp = hp:CreateFontString(nil, "OVERLAY")
	hpp:SetPoint("RIGHT", -4, 0)
	hpp:SetFontObject(GameFontNormal)

	hp.value = hpp
	self.Health = hp
	self.OverrideUpdateHealth = updateHealth

	local name = hp:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 4, 0)
	name:SetPoint("RIGHT", hpp, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, 16)
	name:SetShadowColor(0, 0, 0, 0.9)
	name:SetShadowOffset(1, -1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateName

	if IsAddOnLoaded("oUF_CombatFeedbackText") and unit == "player" then
		local cbft = hp:CreateFontString(nil, "OVERLAY")
		cbft:SetPoint("CENTER", self, "CENTER")
		cbft:SetFontObject(GameFontNormal)
		self.CombatFeedbackText = cbft
		self.CombatFeedbackText.maxAlpha = .6
		-- The only thing we don't ignore is heals.
		self.CombatFeedbackText.ignoreImmune = true
		self.CombatFeedbackText.ignoreDamage = true
		self.CombatFeedbackText.ignoreEnergize = true
		self.CombatFeedbackText.ignoreOther = true
	end

	local ricon = hp:CreateTexture(nil, "OVERLAY")
	ricon:SetHeight(24)
	ricon:SetWidth(24)
	ricon:SetPoint("CENTER", hp, "TOP", 0, 0)
	ricon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = ricon

	if unit ~= "targettarget" and unit ~= "focus" and unit ~= "pet" then
		local pp = CreateFrame("StatusBar")
		pp:SetHeight(8)
		hp:SetHeight(28)
		pp:SetStatusBarTexture(statusbarTexture)
		pp:SetStatusBarColor(.25, .25, .35)

		pp:SetParent(self)
		pp:SetPoint("BOTTOM", self, "BOTTOM")
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")

		self.Power = pp
		self.OverrideUpdatePower = updatePower
	end

	if not unit then
		local debuffs = CreateFrame("Frame", nil, self)
		local s = self.Power:GetHeight() + self.Health:GetHeight()
		debuffs.size = s
		debuffs:SetHeight(s)
		debuffs:SetWidth(s * 3)
		debuffs:SetPoint("LEFT", self, "RIGHT", 4, 0)
		debuffs.initialAnchor = "TOPLEFT"
		debuffs.num = 3 -- max debuffs
		self.PostCreateAuraIcon = auraIcon
		self.Debuffs = debuffs
	elseif unit == "target" then
		local auras = CreateFrame("Frame", nil, self)
		auras.size = self:GetWidth() / 8
		auras:SetHeight(auras.size * 4)
		auras:SetWidth(self:GetWidth())
		auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 2)
		auras.numBuffs = 40
		auras.numDebuffs = 40
		self.Auras = auras
	end

	if unit == "target" then
		local cpoints = self:CreateFontString(nil, "OVERLAY")
		cpoints:SetPoint("RIGHT", self, "LEFT", -8, 1)
		cpoints:SetFont(DAMAGE_TEXT_FONT, 38, "OUTLINE")
		cpoints:SetTextColor(1, 1, 1)
		cpoints:SetJustifyH("RIGHT")
		self.CPoints = cpoints
	end

	if not unit then
		self.Range = true
		self.inRangeAlpha = 1
		self.outsideRangeAlpha = .5
	end

	return self
end

oUF:RegisterStyle("Rabbit", setmetatable({}, {__call = func}))
oUF:SetActiveStyle("Rabbit")

local player = oUF:Spawn("player", "oUF_Player")
player:SetPoint("BOTTOM", -114, 160)
local pet = oUF:Spawn("pet", "oUF_Pet")
pet:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -8)

local target = oUF:Spawn("target", "oUF_Target")
target:SetPoint("BOTTOM", 114, 160)

local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
tot:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -8)
local focus = oUF:Spawn("focus", "oUF_Focus")
focus:SetPoint("TOPLEFT", target, "BOTTOMLEFT", 0, -8)

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", 4, -40)
party:SetAttribute("yOffset", -8)
party:SetAttribute("showParty", true)

local toggleParty = CreateFrame("Frame")
toggleParty:SetScript("OnEvent", function()
	if GetNumRaidMembers() > 0 then
		party:Hide()
	elseif GetNumPartyMembers() > 0 then
		party:Show()
	end
end)
toggleParty:RegisterEvent("PARTY_MEMBERS_CHANGED")
toggleParty:RegisterEvent("PARTY_LEADER_CHANGED")
toggleParty:RegisterEvent("RAID_ROSTER_UPDATE")
toggleParty:RegisterEvent("PLAYER_LOGIN")

