--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG and Lily.
-- by rabbit.magtheridon@gmail.com
--
--]]

--------------------------------------------------------------------------------
-- OPTIONS
-- I added these so that I could easily tweak it myself to try and find the best
-- combination for myself, and the options take up so little space that I just
-- left them in.

-- Where to spawn the first raid group.
local DISTANCE_FROM_LEFT = 16
local DISTANCE_FROM_TOP = -40

-- How many groups to spawn, 0-8.
local MAX_GROUPS_TO_SPAWN = 8
-- The space below and to the side of each unit box.
local UNIT_SPACING = 8

-- [true]: HP and mana bars show as empty when the unit is at full, and fill up
--         as the unit loses hitpoints or mana.
--
-- Setting this to true also hides the dark backdrop.
local REVERSE_STATUSBARS = nil

-- [true]: Color the name/HP text red when someone has aggro instead of the
--         health bar.
local BANZAI_NAMES = nil

-- [percent]: When a unit drops below this set percent, the name in the center
--            will be replaced with a HP indicator. Set it to nil to disable.
local HEALTH_DISPLAY = 90
local HEALTH_DISPLAY_AS = "percent" -- "percent" or "deficit"

-- 0.1-1: Sets the scale of the raid frames.
local SCALE = 1

--------------------------------------------------------------------------------
-- SETTINGS

-- Font used for the center text
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
-- Statusbar texture used for HP and mana bars.
local statusbarTextureVertical = "Interface\\AddOns\\oUF_Rabbit\\media\\smooth"

--------------------------------------------------------------------------------
-- Layout

local inset = { left = -2, right = -2, top = -2, bottom = -2 }
if REVERSE_STATUSBARS then
	inset = { left = 0, right = 0, top = 0, bottom = 0 }
end
local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	insets = inset,
}
local power = oUF.colors.power

local percent = "%d%%"
local deficit = "-%d"
local dead = "DEAD"
local ghost = "GHOST"
local offline = "OFF"

-- Only used if BANZAI_NAMES is set.
local function updateBanzai(self, unit, aggro)
	self.banzaiStatus = aggro == 1 and true or nil
	self:UNIT_NAME_UPDATE(event, unit)
end

local function updateNameRaid(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local textFrame = self.Name
		local u = unit or self.unit
		
		-- We have to set the HP/Power values to 0 here so that when units are
		-- dead and we're running with REVERSE_STATUSBARS, the bars do not fill
		-- up, making the whole thing look silly.
		if UnitIsDead(u) then
			textFrame:SetText(dead)
			self.Health:SetValue(0)
			self.Power:SetValue(0)
			self:SetBackdropColor(0, 0, 0, 0)
		elseif UnitIsGhost(u) then
			textFrame:SetText(ghost)
			self.Health:SetValue(0)
			self.Power:SetValue(0)
			self:SetBackdropColor(0, 0, 0, 0)
		elseif not UnitIsConnected(u) then
			textFrame:SetText(offline)
			self.Health:SetValue(0)
			self.Power:SetValue(0)
			self:SetBackdropColor(0, 0, 0, 0)
		else
			local min, max = UnitHealth(u), UnitHealthMax(u)
			local p = floor(min / max * 100)
			if HEALTH_DISPLAY and p <= HEALTH_DISPLAY then
				if HEALTH_DISPLAY_AS == "percent" then
					textFrame:SetFormattedText(percent, p)
				else
					textFrame:SetFormattedText(deficit, max - min)
				end
			else
				local n = UnitName(u)
				textFrame:SetText(n:sub(1, 3))
			end
			self:SetBackdropColor(0, 0, 0, REVERSE_STATUSBARS and 0 or .9)
		end
		if self.banzaiStatus then
			textFrame:SetTextColor(1, 0, 0)
		else
			local _, c = UnitClass(u)
			local color = RAID_CLASS_COLORS[c]
			if color then
				textFrame:SetTextColor(color.r, color.g, color.b)
			end
		end
	end
end

local function updateHealthRaid(self, event, bar, unit, min, max)
	if REVERSE_STATUSBARS then
		bar:SetValue(max - min)
	end
	local _, c = UnitClass(unit)
	local color = RAID_CLASS_COLORS[c]
	if color then
		bar:SetStatusBarColor(color.r, color.g, color.b)
	end
	self:UNIT_NAME_UPDATE(event, unit)
end

local function updatePowerRaid(self, event, bar, unit, min, max)
	local p = UnitPowerType(unit)
	if p == 0 and min ~= max then
		self.Health:SetWidth(REVERSE_STATUSBARS and 35 or 34)
		if REVERSE_STATUSBARS then
			bar:SetValue(max - min)
		end
		bar:Show()
	else
		self.Health:SetWidth(38)
		bar:Hide()
	end
end

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local function raid(settings, self, unit)
	self.menu = menu

	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, REVERSE_STATUSBARS and 0 or .9)

	if BANZAI_NAMES then
		self.Banzai = updateBanzai
	end

	local hp = CreateFrame("StatusBar")
	hp:SetStatusBarTexture(statusbarTextureVertical)
	hp:SetOrientation("VERTICAL")
	hp:SetParent(self)
	hp:SetPoint("LEFT", self, "LEFT")
	hp:SetPoint("TOP")
	hp:SetPoint("BOTTOM")
	hp:SetWidth(33)
	self.Health = hp
	self.OverrideUpdateHealth = updateHealthRaid

	local mana = power[0]
	local pp = CreateFrame("StatusBar")
	pp:SetStatusBarTexture(statusbarTextureVertical)
	pp:SetStatusBarColor(mana.r, mana.g, mana.b)
	pp:SetOrientation("VERTICAL")
	pp:SetParent(self)
	pp:SetPoint("RIGHT", self, "RIGHT")
	pp:SetPoint("TOP")
	pp:SetPoint("BOTTOM")
	pp:SetWidth(3)
	self.Power = pp
	self.OverrideUpdatePower = updatePowerRaid

	-- We need an overlay to span the name across both hp and power
	local overlay = CreateFrame("Button")
	overlay:SetParent(self)
	overlay:SetAllPoints(self)
	overlay:SetBackdropColor(0, 0, 0, 0)
	overlay:EnableMouse(false)

	if IsAddOnLoaded("oUF_DebuffHighlight") then
		local grid = overlay:CreateTexture(nil, "OVERLAY")
		grid:SetWidth(6)
		grid:SetHeight(6)
		grid:SetTexture("Interface\\AddOns\\oUF_Rabbit\\media\\white16x16")
		grid:SetPoint("BOTTOMLEFT", overlay, "BOTTOMLEFT", 2, -2)
		grid:SetVertexColor(0, 0, 0, 0)
		self.DebuffHighlight = grid
		self.DebuffHighlightAlpha = 1
	end

	local name = overlay:CreateFontString(nil, "OVERLAY")
	name:SetAllPoints(overlay)
	name:SetJustifyH("CENTER")
	name:SetFont(font, 13)
	name:SetShadowColor(0, 0, 0, 0.9)
	name:SetShadowOffset(1, -1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateNameRaid

	local icon = overlay:CreateTexture(nil, "OVERLAY")
	icon:SetHeight(16)
	icon:SetWidth(16)
	icon:SetPoint("TOP", self, 0, 8)
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = icon

	local leader = overlay:CreateTexture(nil, "OVERLAY")
	leader:SetHeight(13)
	leader:SetWidth(13)
	leader:SetPoint("TOPLEFT", self, 3, 6)
	leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	self.Leader = leader

	self.Range = true
	self.inRangeAlpha = 1
	self.outsideRangeAlpha = .5

	return self
end

oUF:RegisterStyle("Rabbit_Raid", setmetatable({
	["initial-width"] = 38,
	["initial-height"] = 32,
}, {__call = raid}))

oUF:SetActiveStyle("Rabbit_Raid")

local raid = {}
for i = 1, MAX_GROUPS_TO_SPAWN do
	local rGroup = oUF:Spawn("header", "oUF_Rabbit_Raid" .. i)
	rGroup:SetAttribute("groupFilter", tostring(i))
	rGroup:SetAttribute("showRaid", true)
	rGroup:SetAttribute("yOffset", -UNIT_SPACING)
	table.insert(raid, rGroup)
	if i == 1 then
		rGroup:SetPoint("TOPLEFT", UIParent, "TOPLEFT", DISTANCE_FROM_LEFT, DISTANCE_FROM_TOP)
	else
		rGroup:SetPoint("TOPLEFT", raid[i-1], "TOPRIGHT", UNIT_SPACING, 0)		
	end
	rGroup:SetScale(SCALE)
	rGroup:Show()
end

